//Pong Version 1.0 by Ignat Drozdov
//Questions/Comments ignat85@yahoo.com

#define USE_TI89              // Produce .89z File

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <string.h>
#include <graph.h>
#include <kbd.h>
#include <system.h>
#include <printf.h>
#include <peekpoke.h>
#include <stdlib.h>

#define ARROWS_ROW 0xfffe     
#define LEFT_KEY 2
#define RIGHT_KEY 8
#define APPS_KEY 6
#define ESC_ROW 0xffbf
#define ESC_KEY 1

void draw_line(int,int,int,int); 
void Draw_Paddle(int x,int y);
void Main_Game();
void Initialize(int human, int calc);
void Move_Calc();
void Move_Ball();

int put_sprite(int x,int y,int sprite[]);//Thanks to Zeljko Juric

//Thanks to Zeljko Juric
void invert_scr(void)
{
  register long *ptr;
  for(ptr=(long*)0x4c00;(unsigned long)ptr<=0x57b8;*ptr++^=-1);
}

int player_x;
int player_y;
int calc_x;
int calc_y;
int ball_x;
int ball_y;
int delta_x;
int crash;
int dir = 1;
int SPEED = 2;
static int ball[] = {
  0b00000000,
  0b00111100,
  0b01111010,
  0b11111001,
  0b11111111,
  0b01111110,
  0b00111100,
  0b00000000};

void _main(void)
{
	clrscr();

	FontSetSys(F_4x6);
	DrawStr(10,20,"Pong v 1.0 Written in C by Ignat Drozdov", A_NORMAL);
	DrawStr(5,27,"Questions, comments:   ignat85@yahoo.com",A_NORMAL);
	DrawStr(50,93,"Press any key to play",A_NORMAL);
	
	ngetchx();
	Initialize(0,0);

	pokeIO(0x600017,0xFA);      //Force timer to work faster
	OSFreeTimer(6);
  OSRegisterTimer(6,SPEED);	
	
	Main_Game();
	pokeIO(0x600017,0xB2);      //Restore timer's normal speed
}

void Main_Game()
{
	OSSetSR(0x0400);
	int key;
	int game = 0;
	int human_score = 0;
	int calc_score = 0;
	
	while(game == 0 && (human_score<10) && (calc_score<10))
	{
		if(_rowread(ESC_ROW)&ESC_KEY) return;
		
		
		put_sprite(ball_x, ball_y,ball);
	  
	  if((put_sprite(ball_x, ball_y, ball)==1) && ((ball_y < 10) || (ball_y > 91)) ) 
	 	{
	 		crash = 1;
	 		int a;
	 		for(a=0;a<=100;a++)invert_scr();
	 		if(ball_y < 10)
	 		{
	 			human_score ++;
	 			Initialize(human_score,calc_score);
	 		}
	 		else
	 		{
	 			calc_score++;
	 			Initialize(human_score,calc_score);
	 		}
	 	}
	  
		
		while (!OSTimerExpired (6));
		OSTimerRestart(6);

		
		key=_rowread(ARROWS_ROW);
		
		Draw_Paddle(player_x, player_y);
		player_x+=((player_x<138)&&(key&RIGHT_KEY))-((player_x>1)&&(key&LEFT_KEY));
		Draw_Paddle(player_x, player_y);
		Move_Ball();
		Move_Calc();
			
	}
}

void Move_Ball()
{
	if((ball_y + dir < 92)||(ball_y + dir>0))
		{
			if(ball_y + 1 > 92)	
			{
				dir = -1;
				if(ball_x-player_x<0) delta_x= -1;
				if((ball_x-player_x>10) && (ball_x-player_x<20)) 
				{
					dir = -2;
					delta_x = 1;
				}
				if((ball_x-player_x<10) && (ball_x-player_x>0)) 
				{
					dir = -2;
					delta_x = -1;
				}
				if(ball_x-player_x == 0) delta_x = 0;
				if(ball_x-player_x>0) delta_x = 1;
			
			}
			if(ball_y - 1 < 8) 
			{
				dir = 1;
				if(ball_x-calc_x<0) delta_x = -1;
				if((ball_x-calc_x>10) && (ball_x-calc_x<20)) 
				{
					dir = 2;
					delta_x = 1;
				}
				if((ball_x-calc_x<10) && (ball_x-player_x>0)) 
				{
					dir = 2;
					delta_x = -1;
				}
				if(ball_x-calc_x>0) delta_x = 1;
			}
		}
		
			put_sprite(ball_x, ball_y,ball);
		
			ball_y+=dir;
			ball_x+=delta_x;
			
			if((ball_x > 143) || (ball_x < -8)) delta_x = -delta_x;
			
			put_sprite(ball_x, ball_y, ball);
}

void Move_Calc()
{
	int cheat = random(50);
		
		if((delta_x>0) && (calc_x+1<139))
		{
			Draw_Paddle(calc_x, calc_y);	
			calc_x++;
			Draw_Paddle(calc_x, calc_y);
		}
		
		if((delta_x<0) && (calc_x-1>0))
		{
			Draw_Paddle(calc_x, calc_y);	
			calc_x--;
			Draw_Paddle(calc_x, calc_y);
		}

	if(cheat == 5)
	{
		if((calc_x < ball_x) && (calc_x + 1 < 139))
		{
			Draw_Paddle(calc_x, calc_y);	
			calc_x++;
			Draw_Paddle(calc_x, calc_y);
		}
		
		if((calc_x > ball_x) && (calc_x - 1 > 0))
		{
			Draw_Paddle(calc_x, calc_y);	
			calc_x--;
			Draw_Paddle(calc_x, calc_y);
		}
	}
}
void Initialize(int human,int calc)
{
	player_x = 70;player_y = 99;calc_x = 70;calc_y = 10;ball_x = 70;ball_y = 50;delta_x = 0;
	crash = 0;
	clrscr();
	DrawLine(0,7,0,99,A_NORMAL);
	DrawLine(159,7,159,99,A_NORMAL);
	Draw_Paddle(player_x, player_y);
	Draw_Paddle(calc_x, calc_y);
	FontSetSys(F_4x6);
	printf("Human: %d                                                    Ti89: %d",human,calc);	
	
	put_sprite(ball_x,ball_y,ball);
}

void Draw_Paddle(int x, int y)
{
	draw_line(x,y,x+20,y);
	draw_line(x,y-1,x+20,y-1);
}

void draw_line(int ri, int ci, int rf, int cf)
{
	DrawLine(ri,ci,rf,cf,A_XOR);
}


int put_sprite(int x,int y,int sprite[])
{
  register long addr=0x4c00+30*y+((x>>3)&0xfffe),collision=0,data;
  register int i,cnt=16-(x&15);
  for(i=0;i<=7;i++,addr+=30)
    collision|=((data=(long)sprite[i]<<cnt)&peek_l(addr)),*(long*)addr^=data;
  return !collision;
}

